#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Ways to install a module
# 
# Install a module from a notebook.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# The module [pymyinstall](http://www.xavierdupre.fr/app/pymyinstall/helpsphinx/) proposes an easy to install module mostly on Windows as it is already quite easy to do it on Linux/Mac through [pip](http://pip.readthedocs.org/en/latest/). There are a couple of ways to install a module and they should be tried in that way:
# 
# * **pip**: pip
# * **wheel**: use also pip but on Windows, it will fetch the wheel file from the location mentioned below (Unofficial...)
# * **github**: source (usually from [github](https://github.com/)), the owner of the source must be specified
# 
# Old way which should not be used anymore:
# 
# * exe: a setup on Windows (only on Windows), replaced by pip in Linux/Max
# * exe_xd: only for Windows, some setups gathered on my website
#     
# [pip](http://pip.readthedocs.org/en/latest/) is great because it deals with dependencies for you. I recommend to use Pyhon 3.4 because that is the first version which includes ``pip``. It takes the modules from [PyPI](https://pypi.python.org/pypi). The only drawback happens on Windows when a module includes [Fortran](http://en.wikipedia.org/wiki/Fortran)/[C](http://en.wikipedia.org/wiki/C_%28programming_language%29)/[C++](http://en.wikipedia.org/wiki/C%2B%2B) files which must be compiled. As opposed to Linux or Mac with [gcc](https://gcc.gnu.org/), there is not an official compiler to handle every package and it has to be installed first. On Windows, it can be done by installing [Visual Studio Express 2010](http://www.visualstudio.com/downloads/download-visual-studio-vs#DownloadFamilies_4) but sometimes, the dependencies can be tricky. That's why it is recommended to install already compiled python extensions. Not every module provides a compiled version but there exists two main ways to get them:
# 
# * [Unofficial Windows Binaries for Python Extension Packages](http://www.lfd.uci.edu/~gohlke/pythonlibs/)
# * [Anaconda](http://continuum.io/)
# 
# [pymysintall](http://www.xavierdupre.fr/app/pymyinstall/helpsphinx/) takes the setup from the first source. The following snippets of code gives an example for each described way to install a module. The setup way only works on Windows.

# ### pip
# 
# Let's try with the following module: [numbers_extractor](https://pypi.python.org/pypi/numbers_extractor).

# In[2]:


from pymyinstall import ModuleInstall


# The following only works if you have enough permissions to do it which is the case if the notebook server is started with admin permissions or if you are using a virtual environment.

# In[3]:


ModuleInstall("numbers_extractor", "pip").install()


# ### wheel (for files *.whl)

# In[4]:


ModuleInstall("numpy", "wheel").install()


# The function checks first if the module was already installed. That's why it displays only True.

# All modules are not available at [Unofficial Windows Binaries for Python Extension Packages](http://www.lfd.uci.edu/~gohlke/pythonlibs/) such as [xgboost](https://github.com/dmlc/xgboost). For this one, ``wheel`` must be replaced by ``wheel2``. The wheel file will be picked from [xavierdupre.fr](http://www.xavierdupre.fr).

# ### github
# 
# [github](https://github.com/) holds the source of most of the open source project. This one is no exception. You can check this page for example [Top 400 Python Projects in Github](http://pythonhackers.com/open-source/). We try here with the module [bottle](https://github.com/defnull/bottle/).

# In[5]:


ModuleInstall("bottle", "github", "defnull").install()


# The function downloads the source from GitHub and install them using the instruction ``python setup.py install``. The function still has to be improved because analyzing the output is always obvious if there are dependencies or error messages. We check again a second call does not install the module again.

# In[6]:


ModuleInstall("bottle", "github", "defnull").install()


# We finally check the module can be imported. It sometimes requires the kernel to restarted.

# In[7]:


import bottle


# The two previous methods download files and create others when some file needs to be uncompressed.

# In[8]:


import os
os.listdir(".")


# In[9]: